//--------------------------------------------------------------------------------------
// File: Tutorial07.fx
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
// Constant Buffer Variables
//--------------------------------------------------------------------------------------
Texture txLeft;
Texture txRight;
int ySize;
bool anaglyph = false;

//#define FLIP_TEXTURE_Y

sampler samLinearLeft  = sampler_state
{
    Texture = <txLeft>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Wrap;
    AddressV = Wrap;
};

sampler samLinearRight  = sampler_state
{
    Texture = <txRight>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VS_INPUT
{
    float4 Pos : POSITION;
    float2 Tex : TEXCOORD;
};

struct PS_INPUT
{
    float4 Pos : POSITION;
    float2 Tex : TEXCOORD0;
};


//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
PS_INPUT VS( VS_INPUT IN )
{
	PS_INPUT OUT = (PS_INPUT)0;

#ifdef FLIP_TEXTURE_Y
    OUT.Tex = float2(IN.Tex.x,(1.0-IN.Tex.y));
#else /* !FLIP_TEXTURE_Y */
    OUT.Tex = IN.Tex.xy;
#endif /* !FLIP_TEXTURE_Y */

	OUT.Pos = IN.Pos;
    return OUT;
}


//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 PS( PS_INPUT input) : COLOR
{
    float4 result;
    if (!anaglyph)
    {
		if ((floor(input.Tex[1]*ySize) % 2) == 1)
		{
			result = tex2D(samLinearLeft, input.Tex );
		}
		else
		{
			result = tex2D(samLinearRight, input.Tex);
	    }
    }
	else
	{
		float4 left = tex2D(samLinearLeft, input.Tex );
		float blackWhiteValueLeft = (0.3f*left.x + 0.59f*left.y + 0.11f*left.z);
		float4 right = tex2D(samLinearRight, input.Tex);
		float blackWhiteValueRight = (0.3f*right.x + 0.59f*right.y + 0.11f*right.z);
		//result = float4( blackWhiteValueLeft, blackWhiteValueRight, blackWhiteValueRight, 1.0f );
		result = float4( 0.7f*left.y+0.3f*left.z, right.y, right.z, 1.0f );
	}
    return result;
}


//--------------------------------------------------------------------------------------
technique Render
{
    pass P0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}

