/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

namespace Tgex
{
    public class Curve2D
    {
        public Curve curveX = new Curve();
        public Curve curveY = new Curve();

        private float m_duration;

        public float Duration
        {
            get { return m_duration; }
            set { m_duration = value; }
        }

        public Curve2D()
        {
            curveX.PreLoop = CurveLoopType.Constant;
            curveX.PostLoop = CurveLoopType.Constant;
            curveY.PreLoop = CurveLoopType.Constant;
            curveY.PostLoop = CurveLoopType.Constant;
            m_duration = 0;
        }
        public void SetPreLoop(CurveLoopType type)
        {
            curveX.PreLoop = type;
            curveY.PreLoop = type;
        }
        public void SetPostLoop(CurveLoopType type)
        {
            curveX.PostLoop = type;
            curveY.PostLoop = type;
        }
        public void addKey(float time, float x, float y)
        {
            curveX.Keys.Add(new CurveKey(time, x));
            curveY.Keys.Add(new CurveKey(time, y));
            if (time > m_duration)
            {
                m_duration = time;
            }
        }
        public void SetTangents()
        {
            CurveKey prev;
            CurveKey current;
            CurveKey next;
            int prevIndex;
            int nextIndex;
            for (int i = 0; i < curveX.Keys.Count; i++)
            {
                prevIndex = i - 1;
                if (prevIndex < 0) prevIndex = i;

                nextIndex = i + 1;
                if (nextIndex == curveX.Keys.Count) nextIndex = i;

                prev = curveX.Keys[prevIndex];
                next = curveX.Keys[nextIndex];
                current = curveX.Keys[i];
                SetCurveKeyTangent(ref prev, ref current, ref next);
                curveX.Keys[i] = current;
            }
            for (int i = 0; i < curveY.Keys.Count; i++)
            {
                prevIndex = i - 1;
                if (prevIndex < 0) prevIndex = i;

                nextIndex = i + 1;
                if (nextIndex == curveY.Keys.Count) nextIndex = i;

                prev = curveY.Keys[prevIndex];
                next = curveY.Keys[nextIndex];
                current = curveY.Keys[i];
                SetCurveKeyTangent(ref prev, ref current, ref next);
                curveY.Keys[i] = current;
            }
        }
        void SetCurveKeyTangent(ref CurveKey prev, ref CurveKey cur, ref CurveKey next)
        {
            float dt = next.Position - prev.Position;
            float dv = next.Value - prev.Value;
            if (Math.Abs(dv) < float.Epsilon)
            {
                cur.TangentIn = 0;
                cur.TangentOut = 0;
            }
            else
            {
                // The in and out tangents should be equal to the slope between the adjacent keys
                cur.TangentIn = dv * (cur.Position - prev.Position) / dt;
                cur.TangentOut = dv * (next.Position - cur.Position) / dt;
            }
        }
        public Vector2 GetPointOnCurve(float time)
        {
            Vector2 point = new Vector2();
            point.X = curveX.Evaluate(time);
            point.Y = curveY.Evaluate(time);
            return point;
        }
    }
}
