﻿/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tgex.Dim4
{
    public class Camera4d : IDisposable
    {
        public Vector4d lookFrom { get; set; }
        public Vector4d lookTo { get; set; }
        public Vector4d upVector { get; set; }
        public Vector4d overVector { get; set; }
        public float xFOV { get; set; }
        public float yFOV { get; set; }
        public float zFOV { get; set; }
        public Matrix4d transformMatrix { get; set; }

        public Camera4d()
        {
            lookFrom = new Vector4d(4, 0, 0, 0);
            lookTo = new Vector4d(0, 0, 0, 0);
            upVector = new Vector4d(0, 1, 0, 0);
            overVector = new Vector4d(0, 0, 1, 0);
            transformMatrix = new Matrix4d();
            xFOV = 0.45f;
            yFOV = 0.45f;
            zFOV = 0.45f;
            setTransformMatrix();
        }
        public void setTransformMatrix()
        {
	        Vector4d xCol;
	        Vector4d yCol;
	        Vector4d zCol;
	        Vector4d hCol;
	        hCol = lookTo - lookFrom;
            hCol.Normalize();

	        xCol = Vector4d.Cross( upVector, overVector, hCol );
            xCol.Normalize();

	        yCol = Vector4d.Cross( overVector, hCol, xCol );
            yCol.Normalize();

            zCol = Vector4d.Cross(hCol, xCol, yCol);

	        transformMatrix.setXCol( xCol );
            transformMatrix.setYCol(yCol);
            transformMatrix.setZCol(zCol);
            transformMatrix.setHCol(hCol);
        }


        #region Dispose
        /// <summary>
        /// Dispose
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Dispose
        /// </summary>
        /// <param name="disposing">Disposing</param>
        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
            }
        }
        #endregion

    }
}
