﻿/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tgex.Dim4
{
    public class Matrix4d :IDisposable
    {
        float xx, xy, xz, xh, xo;
        float yx, yy, yz, yh, yo;
        float zx, zy, zz, zh, zo;
        float hx, hy, hz, hh, ho;

        public Matrix4d()
        {
            xx = 1.0f; xy = 0; xz = 0; xh = 0; xo = 0;
            yx = 0; yy = 1.0f; yz = 0; yh = 0; yo = 0;
            zx = 0; zy = 0; zz = 1.0f; zh = 0; zo = 0;
            hx = 0; hy = 0; hz = 0; hh = 1.0f; ho = 0;
        }
        // set the baby
        public void setXCol(Vector4d x)
        {
            xx = x.x;
            yx = x.y;
            zx = x.z;
            hx = x.h;
        }
        public void setYCol(Vector4d y)
        {
            xy = y.x;
            yy = y.y;
            zy = y.z;
            hy = y.h;
        }
        public void setZCol(Vector4d z)
        {
            xz = z.x;
            yz = z.y;
            zz = z.z;
            hz = z.h;
        }

        public void setHCol(Vector4d h)
        {
            xh = h.x;
            yh = h.y;
            zh = h.z;
            hh = h.h;
        }

        public void setXRow(Vector4d x)
        {
            xx = x.x;
            xy = x.y;
            xz = x.z;
            xh = x.h;
        }
        public void setYRow(Vector4d y)
        {
            yx = y.x;
            yy = y.y;
            yz = y.z;
            yh = y.h;
        }
        public void setZRow(Vector4d z)
        {
            zx = z.x;
            zy = z.y;
            zz = z.z;
            zh = z.h;
        }
        public void setHRow(Vector4d h)
        {
            hx = h.x;
            hy = h.y;
            hz = h.z;
            hh = h.h;
        }

        // get the baby
        public Vector4d getXCol()
        {
            Vector4d vec = new Vector4d(xx, yx, zx, hx);
            return vec;
        }
        public Vector4d getYCol()
        {
            Vector4d vec = new Vector4d(xy, yy, zy, hy);
            return vec;
        }
        public Vector4d getZCol()
        {
            Vector4d vec = new Vector4d(xz, yz, zz, hz);
            return vec;
        }
        public Vector4d getHCol()
        {
            Vector4d vec = new Vector4d(xh, yh, zh, hh);
            return vec;
        }

        public Vector4d getXRow()
        {
            Vector4d vec = new Vector4d(xx, xy, xz, xh);
            return vec;
        }
        public Vector4d getYRow()
        {
            Vector4d vec = new Vector4d(yx, yy, yz, yh);
            return vec;
        }
        public Vector4d getZRow()
        {
            Vector4d vec = new Vector4d(zx, zy, zz, zh);
            return vec;
        }
        public Vector4d getHRow()
        {
            Vector4d vec = new Vector4d(hx, hy, hz, hh);
            return vec;
        }

        // Scale by f in all dimensions
        public void scale(float f)
        {
            xx *= f; xy *= f; xz *= f; xh *= f; xo *= f;
            yx *= f; yy *= f; yz *= f; yh *= f; yo *= f;
            zx *= f; zy *= f; zz *= f; zh *= f; zo *= f;
            hx *= f; hy *= f; hz *= f; hh *= f; ho *= f;
        }

        // Scale along each axis independently
        public void scale(float xf, float yf, float zf, float hf)
        {
            xx *= xf; xy *= xf; xz *= xf; xh *= xf; xo *= xf;
            yx *= yf; yy *= yf; yz *= yf; yh *= yf; yo *= yf;
            zx *= zf; zy *= zf; zz *= zf; zh *= zf; zo *= zf;
            hx *= hf; hy *= hf; hz *= hf; hh *= hf; ho *= hf;
        }

        // Translate the origin
        public void translate(float x, float y, float z, float h)
        {
            xo += x; yo += y; zo += z; ho += h;
        }

        // rotate theta degrees in the xz plane
        public void xzrot(float theta)
        {
            theta *= (float)(Math.PI / 180);
            float ct = (float)Math.Cos(theta);
            float st = (float)Math.Sin(theta);

            float Nxx = (float)(xx * ct + zx * st);
            float Nxy = (float)(xy * ct + zy * st);
            float Nxz = (float)(xz * ct + zz * st);
            float Nxh = (float)(xh * ct + zh * st);
            float Nxo = (float)(xo * ct + zo * st);

            float Nzx = (float)(zx * ct - xx * st);
            float Nzy = (float)(zy * ct - xy * st);
            float Nzz = (float)(zz * ct - xz * st);
            float Nzh = (float)(zh * ct - xh * st);
            float Nzo = (float)(zo * ct - xo * st);

            xx = Nxx; xy = Nxy; xz = Nxz; xh = Nxh; xo = Nxo;
            zx = Nzx; zy = Nzy; zz = Nzz; zh = Nzh; zo = Nzo;
        }

        // rotate theta degrees in the yz plane
        public void yzrot(float theta)
        {
            theta *= (float)(Math.PI / 180);
            float ct = (float)Math.Cos(theta);
            float st = (float)Math.Sin(theta);

            float Nyx = (float)(yx * ct + zx * st);
            float Nyy = (float)(yy * ct + zy * st);
            float Nyz = (float)(yz * ct + zz * st);
            float Nyh = (float)(yh * ct + zh * st);
            float Nyo = (float)(yo * ct + zo * st);

            float Nzx = (float)(zx * ct - yx * st);
            float Nzy = (float)(zy * ct - yy * st);
            float Nzz = (float)(zz * ct - yz * st);
            float Nzh = (float)(zh * ct - yh * st);
            float Nzo = (float)(zo * ct - yo * st);

            yx = Nyx; yy = Nyy; yz = Nyz; yh = Nyh; yo = Nyo;
            zx = Nzx; zy = Nzy; zz = Nzz; zh = Nzh; zo = Nzo;
        }

        // rotate theta degrees in the xy plane
        public void xyrot(float theta)
        {
            theta *= (float)(Math.PI / 180);
            float ct = (float)Math.Cos(theta);
            float st = (float)Math.Sin(theta);

            float Nyx = (float)(yx * ct + xx * st);
            float Nyy = (float)(yy * ct + xy * st);
            float Nyz = (float)(yz * ct + xz * st);
            float Nyh = (float)(yh * ct + xh * st);
            float Nyo = (float)(yo * ct + xo * st);

            float Nxx = (float)(xx * ct - yx * st);
            float Nxy = (float)(xy * ct - yy * st);
            float Nxz = (float)(xz * ct - yz * st);
            float Nxh = (float)(xh * ct - yh * st);
            float Nxo = (float)(xo * ct - yo * st);

            yx = Nyx; yy = Nyy; yz = Nyz; yh = Nyh; yo = Nyo;
            xx = Nxx; xy = Nxy; xz = Nxz; xh = Nxh; xo = Nxo;
        }

        // rotate theta degrees in the xh plane
        public void xhrot(float theta)
        {
            theta *= (float)(Math.PI / 180);
            float ct = (float)Math.Cos(theta);
            float st = (float)Math.Sin(theta);

            float Nhx = (float)(hx * ct + xx * st);
            float Nhy = (float)(hy * ct + xy * st);
            float Nhz = (float)(hz * ct + xz * st);
            float Nhh = (float)(hh * ct + xh * st);
            float Nho = (float)(ho * ct + xo * st);

            float Nxx = (float)(xx * ct - hx * st);
            float Nxy = (float)(xy * ct - hy * st);
            float Nxz = (float)(xz * ct - hz * st);
            float Nxh = (float)(xh * ct - hh * st);
            float Nxo = (float)(xo * ct - ho * st);

            hx = Nhx; hy = Nhy; hz = Nhz; hh = Nhh; ho = Nho;
            xx = Nxx; xy = Nxy; xz = Nxz; xh = Nxh; xo = Nxo;
        }

        // rotate theta degrees in the yh plane
        public void yhrot(float theta)
        {
            theta *= (float)(Math.PI / 180);
            float ct = (float)Math.Cos(theta);
            float st = (float)Math.Sin(theta);

            float Nhx = (float)(hx * ct + yx * st);
            float Nhy = (float)(hy * ct + yy * st);
            float Nhz = (float)(hz * ct + yz * st);
            float Nhh = (float)(hh * ct + yh * st);
            float Nho = (float)(ho * ct + yo * st);

            float Nyx = (float)(yx * ct - hx * st);
            float Nyy = (float)(yy * ct - hy * st);
            float Nyz = (float)(yz * ct - hz * st);
            float Nyh = (float)(yh * ct - hh * st);
            float Nyo = (float)(yo * ct - ho * st);

            hx = Nhx; hy = Nhy; hz = Nhz; hh = Nhh; ho = Nho;
            yx = Nyx; yy = Nyy; yz = Nyz; yh = Nyh; yo = Nyo;
        }

        // Multiply this matrix by a second: M = M*R
        public void mult(Matrix4d rhs)
        {
            float lxx = xx * rhs.xx + yx * rhs.xy + zx * rhs.xz + hx * rhs.xh;
            float lxy = xy * rhs.xx + yy * rhs.xy + zy * rhs.xz + hy * rhs.xh;
            float lxz = xz * rhs.xx + yz * rhs.xy + zz * rhs.xz + hz * rhs.xh;
            float lxh = xh * rhs.xx + yh * rhs.xy + zh * rhs.xz + hh * rhs.xh;
            float lxo = xo * rhs.xx + yo * rhs.xy + zo * rhs.xz + ho * rhs.xh + rhs.xo;

            float lyx = xx * rhs.yx + yx * rhs.yy + zx * rhs.yz + hx * rhs.yh;
            float lyy = xy * rhs.yx + yy * rhs.yy + zy * rhs.yz + hy * rhs.yh;
            float lyz = xz * rhs.yx + yz * rhs.yy + zz * rhs.yz + hz * rhs.yh;
            float lyh = xh * rhs.yx + yh * rhs.yy + zh * rhs.yz + hh * rhs.yh;
            float lyo = xo * rhs.yx + yo * rhs.yy + zo * rhs.yz + ho * rhs.yh + rhs.yo;

            float lzx = xx * rhs.zx + yx * rhs.zy + zx * rhs.zz + hx * rhs.zh;
            float lzy = xy * rhs.zx + yy * rhs.zy + zy * rhs.zz + hy * rhs.zh;
            float lzz = xz * rhs.zx + yz * rhs.zy + zz * rhs.zz + hz * rhs.zh;
            float lzh = xh * rhs.zx + yh * rhs.zy + zh * rhs.zz + hh * rhs.zh;
            float lzo = xo * rhs.zx + yo * rhs.zy + zo * rhs.zz + ho * rhs.zh + rhs.zo;

            float lhx = xx * rhs.hx + yx * rhs.hy + zx * rhs.hz + hx * rhs.hh;
            float lhy = xy * rhs.hx + yy * rhs.hy + zy * rhs.hz + hy * rhs.hh;
            float lhz = xz * rhs.hx + yz * rhs.hy + zz * rhs.hz + hz * rhs.hh;
            float lhh = xh * rhs.hx + yh * rhs.hy + zh * rhs.hz + hh * rhs.hh;
            float lho = xo * rhs.hx + yo * rhs.hy + zo * rhs.hz + ho * rhs.hh + rhs.ho;

            xx = lxx; xy = lxy; xz = lxz; xh = lxh; xo = lxo;
            yx = lyx; yy = lyy; yz = lyz; yh = lyh; yo = lyo;
            zx = lzx; zy = lzy; zz = lzz; zh = lzh; zo = lzo;
            hx = lhx; hy = lhy; hz = lhz; hh = lhh; ho = lho;
        }

        // Reinitialize to the unit matrix
        public void unit()
        {
            xx = 1; xy = 0; xz = 0; xh = 0; xo = 0;
            yx = 0; yy = 1; yz = 0; yh = 0; yo = 0;
            zx = 0; zy = 0; zz = 1; zh = 0; zo = 0;
            hx = 0; hy = 0; hz = 0; hh = 1; ho = 0;
        }

        /* Transform nvert points from v into tv.  v contains the input
           coordinates in doubleing point.  Three successive entries in
           the array constitute a point.  tv ends up holding the transformed
           points as integers; three successive entries per point */
        public void transform(float[] v, int[] tv, int nvert)
        {
            float lxx = xx, lxy = xy, lxz = xz, lxh = xh, lxo = xo;
            float lyx = yx, lyy = yy, lyz = yz, lyh = yh, lyo = yo;
            float lzx = zx, lzy = zy, lzz = zz, lzh = zh, lzo = zo;
            float lhx = hx, lhy = hy, lhz = hz, lhh = hh, lho = ho;
            for (int i = nvert * 4; (i -= 4) >= 0; )
            {
                float x = v[i];
                float y = v[i + 1];
                float z = v[i + 2];
                float h = v[i + 3];
                tv[i] = (int)(x * lxx + y * lxy + z * lxz + h * lxh + lxo);
                tv[i + 1] = (int)(x * lyx + y * lyy + z * lyz + h * lyh + lyo);
                tv[i + 2] = (int)(x * lzx + y * lzy + z * lzz + h * lzh + lzo);
                tv[i + 3] = (int)(x * lhx + y * lhy + z * lhz + h * lhh + lho);
            }
        }

        public void transform(ref List<Vector4d> vertices)
        {
            float lxx = xx, lxy = xy, lxz = xz, lxh = xh, lxo = xo;
            float lyx = yx, lyy = yy, lyz = yz, lyh = yh, lyo = yo;
            float lzx = zx, lzy = zy, lzz = zz, lzh = zh, lzo = zo;
            float lhx = hx, lhy = hy, lhz = hz, lhh = hh, lho = ho;

            for (int i = 0; i < vertices.Count; i++)
            {
                float x = vertices[i].x;
                float y = vertices[i].y;
                float z = vertices[i].z;
                float h = vertices[i].h;
                vertices[i].Set(
                    ((x * lxx + y * lxy + z * lxz + h * lxh + lxo)),
                    ((x * lyx + y * lyy + z * lyz + h * lyh + lyo)),
                    ((x * lzx + y * lzy + z * lzz + h * lzh + lzo)),
                    ((x * lhx + y * lhy + z * lhz + h * lhh + lho)));
            }
        }

        public void transform(ref List<Vector4d> verticesOut, List<Vector4d> verticesIn)
        {
            float lxx = xx, lxy = xy, lxz = xz, lxh = xh, lxo = xo;
            float lyx = yx, lyy = yy, lyz = yz, lyh = yh, lyo = yo;
            float lzx = zx, lzy = zy, lzz = zz, lzh = zh, lzo = zo;
            float lhx = hx, lhy = hy, lhz = hz, lhh = hh, lho = ho;

            for (int i = 0; i < verticesIn.Count; i++)
            {
                float x = verticesIn[i].x;
                float y = verticesIn[i].y;
                float z = verticesIn[i].z;
                float h = verticesIn[i].h;
                Vector4d newVal = new Vector4d(
                    ((x * lxx + y * lxy + z * lxz + h * lxh + lxo)),
                    ((x * lyx + y * lyy + z * lyz + h * lyh + lyo)),
                    ((x * lzx + y * lzy + z * lzz + h * lzh + lzo)),
                    ((x * lhx + y * lhy + z * lhz + h * lhh + lho)));
                verticesOut[i] = newVal;
            }
        }

        #region Disposing
        /// <summary>
        /// Dispose
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Dispose
        /// </summary>
        /// <param name="disposing">Disposing</param>
        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
            }
        }
        #endregion

    }

}
