﻿/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tgex.Dim4
{
    public struct Vector4d
    {
        public float x { get; set; }
        public float y { get; set; }
        public float z { get; set; }
        public float h { get; set; }

        public Vector4d(float _x, float _y, float _z, float _h)
            : this()
        {
            x = _x;
            y = _y;
            z = _z;
            h = _h;
        }

        public static Vector4d operator +(Vector4d v1, Vector4d v2)
        {
            return new Vector4d(
                v1.x + v2.x, v1.y + v2.y, v1.z + v2.z, v1.h + v2.h);
        }
        public static Vector4d operator -(Vector4d v1, Vector4d v2)
        {
            return new Vector4d(
                v1.x - v2.x, v1.y - v2.y, v1.z - v2.z, v1.h - v2.h);
        }
        public static Vector4d operator *(Vector4d v, float s)
        {
            return new Vector4d(
                v.x * s, v.y * s, v.z * s, v.h * s);
        }
        public static Vector4d operator *(float s, Vector4d v)
        {
            return new Vector4d(
                v.x * s, v.y * s, v.z * s, v.h * s);
        }
        public static float operator *(Vector4d v1, Vector4d v2)
        {
            return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z + v1.h * v2.h;
        }
        public static Vector4d operator /(Vector4d v1, Vector4d v2)
        {
            return new Vector4d(
                v1.x * v2.y - v1.y * v2.x,
                v1.y * v2.z - v1.z * v2.y,
                v1.z * v2.x - v1.x * v2.z,
                1);
        }
        public override bool Equals(Object obj)
        {
            return ((x == ((Vector4d)obj).x) 
                && (y == ((Vector4d)obj).y)
                && (z == ((Vector4d)obj).z)
                && (h == ((Vector4d)obj).h));
        }
        public override int GetHashCode()
        {
            return (int)x;
        }
        public static bool operator ==(Vector4d v1, Vector4d v2)
        {
            return v1.Equals(v2);
        }
        public static bool operator !=(Vector4d v1, Vector4d v2)
        {
            return !v1.Equals(v2);
        }
        public void Normalize()
        {
            float div = (float)
                Math.Sqrt(x * x + y * y + z * z + h * h);
            x /= div;
            y /= div;
            z /= div;
            h /= div;
        }
        public static Vector4d Cross(Vector4d u, Vector4d v, Vector4d w)
        {
            float A, B, C, D, E, F;
            Vector4d result = new Vector4d();

            A = (v.x * w.y) - (v.y * w.x);
            B = (v.x * w.z) - (v.z * w.x);
            C = (v.x * w.h) - (v.h * w.x);
            D = (v.y * w.z) - (v.z * w.y);
            E = (v.y * w.h) - (v.h * w.y);
            F = (v.z * w.h) - (v.h * w.z);

            result.x =   (u.y * F) - (u.z * E) + (u.h * D);
            result.y = - (u.x * F) + (u.z * C) - (u.h * B);
            result.z =   (u.x * E) - (u.y * C) + (u.h * A);
            result.h = - (u.x * D) + (u.y * B) - (u.z * A);

            return result;

        }
        public float Length()
        {
            return (float)
                Math.Sqrt(x * x + y * y + z * z + h * h);
        }
        public void Set(float _x, float _y, float _z, float _h)
        {
            x = _x;
            y = _y;
            z = _z;
            h = _h;
        }

    }
}
