﻿/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace Tgex
{
    public class DynobFactory
    {
        // create singleton
        static readonly DynobFactory instance = new DynobFactory();
        /// <summary>
        /// Get instance of singleton.
        /// </summary>
        public static DynobFactory Instance
        {
            get
            {
                return instance;
            }
        }
        /// <summary>
        /// Explicit static constructor to tell C# compiler
        /// not to mark type as beforefieldinit
        /// </summary>
        static DynobFactory()
        {
        }
        /// <summary>
        /// Constructor
        /// </summary>
        DynobFactory()
        {
        }
        private GraphicsDeviceManager m_graphicsManager;
        public GraphicsDeviceManager GraphicsManager
        {
            get { return m_graphicsManager; }
            set { m_graphicsManager = value; }
        }
        protected ContentManager m_ContentManager;
        /// <summary>
        /// Content mananger
        /// </summary>
        public ContentManager ContentManager
        {
            get { return m_ContentManager; }
            set { m_ContentManager = value; }
        }
	
        Dictionary<String, XmlElement> m_blueprint = new Dictionary<string, XmlElement>();
        public void AddBlueprint(XmlElement blueprint)
        {
            String name = blueprint.GetAttribute("name");
            m_blueprint.Add(name, blueprint);
        }
        // create from name
        public Dynob Create(String name)
        {
            XmlElement blueprint = m_blueprint[name];
            Dynob dynob = new Dynob();
            // create and add properties
            XmlNode properties = blueprint.GetElementsByTagName("PropertyList").Item(0);
            if (properties != null)
            {
                foreach (XmlNode property in properties.ChildNodes)
                {
                    String typeString = "Tgex." + property.Attributes.GetNamedItem("class").Value;
                    Type t = Type.GetType(typeString);
                    if (t != null)
                    {
                        Property p = (Property)Activator.CreateInstance(t);
                        p.Parse(property);
                        dynob.AddProperty(p);
                    }
                }
            }
            // create modules without parameters
            XmlNode modules = blueprint.GetElementsByTagName("ModuleList").Item(0);
            if (modules != null)
            {
                foreach (XmlNode module in modules.ChildNodes)
                {
                    String typeString = "Tgex." + module.Attributes.GetNamedItem("class").Value;
                    Type t = Type.GetType(typeString);
                    if (t != null)
                    {
                        Module m = (Module)Activator.CreateInstance(t);
                        m.Name = module.Attributes.GetNamedItem("name").Value;
                        dynob.AddModule(m);
                        m.Register(dynob);
                    }
                }
            }
            // create modifier
            XmlNode modifierList = blueprint.GetElementsByTagName("ModifierList").Item(0);
            if (modifierList != null)
            {
                foreach (XmlNode modifier in modifierList.ChildNodes)
                {
                    String modifierName = modifier.Attributes.GetNamedItem("name").Value;
                    String typeString = "Tgex." + modifier.Attributes.GetNamedItem("class").Value;
                    Modifier modifierInstance = null;
                    Type t = Type.GetType(typeString);
                    if (t != null)
                    {
                        modifierInstance = (Modifier)Activator.CreateInstance(t);
                        modifierInstance.Name = modifierName;
                        foreach (XmlNode parameterNode in modifier.ChildNodes)
                        {
                            modifierInstance.AddParameter(InstanceParameter.GetParameter(
                                parameterNode, dynob));
                        }
                    }
                    dynob.AddModifier(modifierInstance);
                }
            }
            // read conditions to a dictionary
            Dictionary<String, ICondition> conditions = dynob.ConditionList;
            XmlNode conditionList = blueprint.GetElementsByTagName("ConditionList").Item(0);
            if (conditionList != null)
            {
                foreach (XmlNode condition in conditionList.ChildNodes)
                {
                    String conditionName = condition.Attributes.GetNamedItem("name").Value;
                    String typeString = "Tgex." + condition.Attributes.GetNamedItem("class").Value;
                    ICondition condInstance = null;
                    Type t = Type.GetType(typeString);
                    if (t != null)
                    {
                        condInstance = (ICondition)Activator.CreateInstance(t);
                        foreach (XmlNode parameterNode in condition.ChildNodes)
                        {
                            condInstance.AddParameter(InstanceParameter.GetParameter(
                                parameterNode, dynob));
                        }
                    }
                    conditions.Add(conditionName, condInstance);
                }
            }
            // read actions to a dictionary
            Dictionary<String, IAction> actions = dynob.ActionList;
            XmlNode actionList = blueprint.GetElementsByTagName("ActionList").Item(0);
            if (actionList != null)
            {
                foreach (XmlNode action in actionList.ChildNodes)
                {
                    String actionName = action.Attributes.GetNamedItem("name").Value;
                    String typeString = "Tgex." + action.Attributes.GetNamedItem("class").Value;
                    IAction actionInstance = null;
                    Type t = Type.GetType(typeString);
                    if (t != null)
                    {
                        actionInstance = (IAction)Activator.CreateInstance(t);
                        actionInstance.Name = actionName;
                        foreach (XmlNode parameterNode in action.ChildNodes)
                        {
                            actionInstance.AddParameter(InstanceParameter.GetParameter(
                                parameterNode, dynob));
                        }
                    }
                    actions.Add(actionName, actionInstance);
                }
            }
            // read trigger
            XmlNode triggerList = blueprint.GetElementsByTagName("TriggerList").Item(0);
            if (triggerList != null)
            {
                foreach (XmlNode trigger in triggerList.ChildNodes)
                {
                    String triggerName = trigger.Attributes.GetNamedItem("name").Value;
                    String typeString = "Tgex." + trigger.Attributes.GetNamedItem("class").Value;
                    IAction action = actions[trigger.Attributes.GetNamedItem("action").Value];
                    ICondition condition = conditions[trigger.Attributes.GetNamedItem("condition").Value];
                    Trigger t = new Trigger();
                    t.Name = triggerName;
                    t.AddAction(action);
                    t.SetCondition(condition);
                    dynob.AddTrigger(t);
                }
            }
            // now create parameters for modules
            foreach (XmlNode module in modules.ChildNodes)
            {
                Module m = dynob.GetModuleByName(module.Attributes.GetNamedItem("name").Value);
                // get and set parameters
                foreach (XmlNode parameterNode in module.ChildNodes)
                {
                    m.AddParameter(InstanceParameter.GetParameter(
                        parameterNode, dynob));
                }
            }
            dynob.Initialize();
            return dynob;
        }
    }
}
