﻿/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

namespace Tgex
{
    public struct InstanceParameter
    {
        public Type type;
        public String name;
        public Object value;

        public static InstanceParameter GetParameter(XmlNode node, Dynob dynob)
        {
            InstanceParameter par = new InstanceParameter();
            String typeString = node.Attributes.GetNamedItem("type").Value;
            par.type = Type.GetType("Tgex." + typeString);
            XmlNode nameNode = node.Attributes.GetNamedItem("name");
            if (nameNode != null)
                par.name = nameNode.Value;
            String value = null;
            XmlNode valueNode = node.Attributes.GetNamedItem("value");
            if (valueNode != null)
            {
                value = node.Attributes.GetNamedItem("value").Value;
            }
            // find in dynob
            switch (typeString)
            {
                case "Property":
                    par.value = dynob.GetPropertyByName(value);
                    break;
                case "Module":
                    par.value = dynob.GetModuleByName(value);
                    break;
                case "Modifier":
                    par.value = dynob.GetModifierByName(value);
                    break;
                case "Trigger":
                    par.value = dynob.GetTriggerByName(value);
                    break;
                case "Action":
                    par.value = dynob.ActionList[value];
                    break;
                case "Condition":
                    par.value = dynob.ConditionList[value];
                    break;
                case "Vector2":
                    par.value = value.ParseVector2();
                    break;
                case "Vector3":
                    par.value = value.ParseVector3();
                    break;
                case "float":
                    par.value = float.Parse(value, System.Globalization.CultureInfo.InvariantCulture.NumberFormat);
                    break;
                case "int":
                    par.value = Int32.Parse(value);
                    break;
                case "bool":
                    par.value = bool.Parse(value);
                    break;
                case "string":
                    par.value = value;
                    break;
                case "Texture2D":
                    par.value = value;
                    break;
                case "ParameterArray":
                    ParameterArray array = new ParameterArray();
                    foreach (XmlNode parameterNode in node.ChildNodes)
                    {
                        array.List.Add(InstanceParameter.GetParameter(
                            parameterNode, dynob));
                    }
                    par.value = array;
                    break;
            }
            return par;
        }
    }
}
