/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Globalization;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace Tgex
{
    public class GameSettings
    {
        public float eyeDistance = 0.07f;
        public bool switchLeftRight = false;
        public float fieldOfView = MathHelper.ToRadians(70);

        // antialias
        public bool antiAlias = false;
        public MultiSampleType multiSampleType = MultiSampleType.None;
        public int multiSampleQuality = 0;

        public int windowWidth = 800;
        public int windowHeight = 600;
        public int windowPositionX = 0;
        public int windowPositionY = 0;
        public bool anaglyph = true;
        public bool fullscreen = false;
#if WINDOWS || XBOX
        public StereoViewMode stereoMode = StereoViewMode.None;
#endif

        /// <summary>
        /// Read settings from xml file
        /// </summary>
        public void ReadSettings()
        {
            // read settings
            // get config from xml file
            try
            {
                XmlDocument xmlFile = new XmlDocument();
                NumberFormatInfo nfi = CultureInfo.InvariantCulture.NumberFormat;
                xmlFile.Load("./content/settings.xml");
                XmlNodeList settingsNodes = xmlFile.GetElementsByTagName("Stereo");
                XmlAttributeCollection attributes = settingsNodes[0].Attributes;
                foreach (XmlAttribute attribute in attributes)
                {
                    switch (attribute.Name)
                    {
                        case "eyeDistance":
                            eyeDistance = float.Parse(attribute.Value, nfi);
                            break;
                        case "switchLeftRight":
                            switchLeftRight = bool.Parse(attribute.Value);
                            break;
                        case "fieldOfView":
                            fieldOfView =
                                MathHelper.ToRadians(float.Parse(attribute.Value, nfi));
                            break;
                        case "antiAlias":
                            antiAlias = bool.Parse(attribute.Value);
                            break;
                        case "width":
                            windowWidth = Int32.Parse(attribute.Value);
                            break;
                        case "height":
                            windowHeight = Int32.Parse(attribute.Value);
                            break;
                        case "fullscreen":
                            fullscreen = bool.Parse(attribute.Value);
                            break;
#if WINDOWS || XBOX
                        case "stereoMode":
                            if ((attribute.Value) == "horizontalSpan")
                                stereoMode = StereoViewMode.HorizontalSpan;
                            else if ((attribute.Value) == "none")
                                stereoMode = StereoViewMode.None;
                            else if ((attribute.Value) == "fieldInterlaced")
                                stereoMode = StereoViewMode.FieldInterlaced;
                            else if ((attribute.Value) == "lineInterlaced")
                                stereoMode = StereoViewMode.LineInterlaced;
                            break;
#endif
                        case "anaglyph":
                            anaglyph = bool.Parse(attribute.Value);
                            break;
                        case "windowPosition":
                            String[] values = attribute.Value.Split(',');
                            windowPositionX = Int32.Parse(values[0]);
                            windowPositionY = Int32.Parse(values[1]);
                            break;
                    }
                }
            }
            catch (Exception)
            {
                ErrorHandler.Report("Error reading settings file!");
            }
        }
    }
}
