/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Tgex.Graphics
{
    public class HorizontalSpanRenderWindow : RenderWindow
    {
        protected Viewport m_leftViewport;
        protected Viewport m_rightViewport;

        public HorizontalSpanRenderWindow(VRGame game)
            : base(game)
        {
#if WINDOWS
            m_mainWin.StartPosition =
                System.Windows.Forms.FormStartPosition.Manual;
            m_mainWin.Location = new System.Drawing.Point(0, 0);
            m_mainWin.Height = m_height;
            m_mainWin.Width = m_width * 2;
#endif
            // prepare graphics settings
            if (m_antiAlias) m_game.GraphicsManager.PreferMultiSampling = true;
            m_game.GraphicsManager.PreferredBackBufferWidth = m_width * 2;
            m_game.GraphicsManager.IsFullScreen = false;

        }

        public override void Initialize()
        {
            base.Initialize();

            // set stereo viewports
            m_defaultViewport = m_game.GraphicsDevice.Viewport;
#if WINDOWS
            m_mainWin.Location = new System.Drawing.Point(0, 0);
            m_mainWin.ControlBox = false;
            m_mainWin.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
            m_mainWin.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
#endif
            m_leftViewport = m_defaultViewport;
            m_rightViewport = m_defaultViewport;
            m_leftViewport.Width = m_leftViewport.Width / 2;
            m_rightViewport.Width = m_rightViewport.Width / 2;
            m_rightViewport.X = m_leftViewport.Width + 1;

            GraphicsDeviceReset(null, EventArgs.Empty);
        }

        /// <summary>
        /// Graphics device reset handler.
        /// </summary>
        public override void GraphicsDeviceReset(object sender, EventArgs e)
        {
            // Update width and height
            m_width = m_game.GraphicsDevice.Viewport.Width / 2;

            m_height = m_game.GraphicsDevice.Viewport.Height;
            m_game.Camera.AspectRatio = (float)m_width / (float)m_height;
            m_game.Camera.UpdateProjectionMatrix();

            // Re-Set device
            // Restore z buffer state
            m_game.GraphicsDevice.RenderState.DepthBufferEnable = true;
            m_game.GraphicsDevice.RenderState.DepthBufferWriteEnable = true;
            // Set u/v addressing back to wrap
            m_game.GraphicsDevice.SamplerStates[0].AddressU = TextureAddressMode.Wrap;
            m_game.GraphicsDevice.SamplerStates[0].AddressV = TextureAddressMode.Wrap;

            // Set 128 and greate alpha compare for Model.Render
            m_game.GraphicsDevice.RenderState.ReferenceAlpha = 128;
            m_game.GraphicsDevice.RenderState.AlphaFunction = CompareFunction.Greater;
#if WINDOWS
            m_mainWin.Height = m_height;
            m_mainWin.Width = m_width * 2;
#endif
        }

        override public void UpdateStereo()
        {
            base.UpdateStereo();

            if (m_isLeft != m_switchLeftRight)
            {
                m_game.GraphicsManager.GraphicsDevice.Viewport = m_leftViewport;
                m_game.GraphicsDevice.Viewport = m_leftViewport;
            }
            else
            {
                m_game.GraphicsManager.GraphicsDevice.Viewport = m_rightViewport;
                m_game.GraphicsDevice.Viewport = m_rightViewport;
            }
        }
        /// <summary>
        /// Draw
        /// </summary>
        /// <param name="gameTime">Game time</param>
        public override void Draw(GameTime gameTime)
        {
            // left eye
            m_isLeft = true;
            UpdateStereo();
            m_game.DrawScene(gameTime);
            // right eye
            m_isLeft = false;
            UpdateStereo();
            m_game.DrawScene(gameTime);
        }
    }
}
