/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Tgex.Graphics
{
    public abstract class RenderWindow
    {
        protected VRGame m_game;
        protected Viewport m_defaultViewport;
        #if WINDOWS
        protected System.Windows.Forms.Form m_mainWin;
        #endif
        protected bool m_isLeft = true;
        protected bool m_switchLeftRight = false;

        protected bool m_antiAlias;
        public bool AntiAlias
        {
            get { return m_antiAlias; }
        }
        protected MultiSampleType m_multiSampleType;

        public MultiSampleType MultiSampleType
        {
            get { return m_multiSampleType; }
            set { m_multiSampleType = value; }
        }
        protected int m_multiSampleQuality;

        public int MultiSampleQuality
        {
            get { return m_multiSampleQuality; }
            set { m_multiSampleQuality = value; }
        }
        protected int m_width;

        public int Width
        {
            get { return m_width; }
            set { m_width = value; }
        }
        protected int m_height;

        public int Height
        {
            get { return m_height; }
            set { m_height = value; }
        }
        protected bool m_fullscreen;

        public bool Fullscreen
        {
            get { return m_fullscreen; }
            set { m_fullscreen = value; }
        }

        public RenderWindow(VRGame game)
        {
            m_game = game;
            m_width = GraphicsAdapter.DefaultAdapter.CurrentDisplayMode.Width;
            m_height = GraphicsAdapter.DefaultAdapter.CurrentDisplayMode.Height;
            // from settings
            m_fullscreen = m_game.GameSettings.fullscreen;
            m_antiAlias = m_game.GameSettings.antiAlias;
            m_multiSampleType = m_game.GameSettings.multiSampleType;
            m_multiSampleQuality = m_game.GameSettings.multiSampleQuality;
            m_switchLeftRight = m_game.GameSettings.switchLeftRight;

            // get windows from handle and set window properties
            m_mainWin = (System.Windows.Forms.Form)
                System.Windows.Forms.Form.FromHandle(m_game.Window.Handle);

            m_game.GraphicsManager.PreferredBackBufferWidth = m_width;
            m_game.GraphicsManager.PreferredBackBufferHeight = m_height;
            m_game.GraphicsManager.IsFullScreen = m_fullscreen;

            m_game.GraphicsManager.PreparingDeviceSettings +=
                new EventHandler<PreparingDeviceSettingsEventArgs>(
                    GraphicsPrepareDevice);
        }

        void GraphicsPrepareDevice(object sender, PreparingDeviceSettingsEventArgs e)
        {
            if (m_antiAlias)
            {
                int quality = 0;
                GraphicsAdapter adapter = e.GraphicsDeviceInformation.Adapter;
                SurfaceFormat format = adapter.CurrentDisplayMode.Format;
                // Check for 4xAA
                if (adapter.CheckDeviceMultiSampleType(DeviceType.Hardware, format,
                    false, MultiSampleType.FourSamples, out quality))
                {
                    // even if a greater quality is returned, we only want quality 1
                    e.GraphicsDeviceInformation.PresentationParameters.MultiSampleQuality = 1;
                    e.GraphicsDeviceInformation.PresentationParameters.MultiSampleType =
                        MultiSampleType.FourSamples;
                    m_multiSampleType = MultiSampleType.FourSamples;
                }
                // Check for 2xAA
                else if (adapter.CheckDeviceMultiSampleType(DeviceType.Hardware, format,
                    false, MultiSampleType.TwoSamples, out quality))
                {
                    // even if a greater quality is returned, we only want quality 1
                    e.GraphicsDeviceInformation.PresentationParameters.MultiSampleQuality = 1;
                    e.GraphicsDeviceInformation.PresentationParameters.MultiSampleType =
                        MultiSampleType.TwoSamples;
                    m_multiSampleType = MultiSampleType.TwelveSamples;
                }
                else
                {
                    m_multiSampleType = MultiSampleType.None;
                }
                m_multiSampleQuality = quality;
            }
        }

        /// <summary>
        /// This is calles form the game initialize function
        /// </summary>
        public virtual void Initialize()
        {
            // Update resolution if it changes
            m_game.GraphicsManager.DeviceReset += new EventHandler(GraphicsDeviceReset);
            //GraphicsDeviceReset(null, EventArgs.Empty);
        }

        public abstract void GraphicsDeviceReset(object sender, EventArgs e);

        public virtual  void UpdateStereo()
        {
            if (m_isLeft)
            {
                m_game.Camera.CurrentEye = Camera.Eye.Left;
                m_game.Camera.UpdateViewMatrix();
            }
            else
            {
                m_game.Camera.CurrentEye = Camera.Eye.Right;
                m_game.Camera.UpdateViewMatrix();
            }
        }
        /// <summary>
        /// Has to call draw scene from game class!
        /// </summary>
        /// <param name="gameTime"></param>
        public abstract void Draw(GameTime gameTime);

        public virtual void SetRenderTarget(RenderTarget2D renderTarget)
        {
            m_game.GraphicsDevice.SetRenderTarget(0, renderTarget);
        }

        public virtual void ResetRenderTarget()
        {
            m_game.GraphicsDevice.SetRenderTarget(0, null);
        }
    }
}
