/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Globalization;

namespace Tgex
{
    public enum SpeedState
    {
        slow,
        normal,
        fast
    }
    public class LinearGrammerModifierDesc
    {
        public float stepSizeSlow;
        public float stepSizeNormal;
        public float stepSizeFast;

        public float slow2slow;
        public float slow2normal;
        public float slow2turn;

        public float normal2slow;
        public float normal2normal;
        public float normal2fast;

        public float fast2normal;
        public float fast2fast;

        public void InitDefault()
        {
            stepSizeSlow = 0.003f;
            stepSizeNormal = 0.006f;
            stepSizeFast = 0.009f;

            slow2slow = 0.2f;
            slow2normal = 0.6f;
            slow2turn = 0.2f;

            normal2slow = 0.2f;
            normal2normal = 0.7f;
            normal2fast = 0.1f;

            fast2normal = 0.2f;
            fast2fast = 0.8f;
        }
        public void SetupFromXml(XmlNode node)
        {
            NumberFormatInfo nfi = CultureInfo.InvariantCulture.NumberFormat;
            XmlAttributeCollection attributes = node.Attributes;
            foreach (XmlAttribute attribute in attributes)
            {
                switch (attribute.Name)
                {
                    case "stepSizeSlow":
                        stepSizeSlow = float.Parse(attribute.Value, nfi);
                        break;
                    case "stepSizeNormal":
                        stepSizeNormal = float.Parse(attribute.Value, nfi);
                        break;
                    case "stepSizeFast":
                        stepSizeFast = float.Parse(attribute.Value, nfi);
                        break;

                    case "slow2slow":
                        slow2slow = float.Parse(attribute.Value, nfi);
                        break;
                    case "slow2normal":
                        slow2normal = float.Parse(attribute.Value, nfi);
                        break;
                    case "slow2turn":
                        slow2turn = float.Parse(attribute.Value, nfi);
                        break;

                    case "normal2slow":
                        normal2slow = float.Parse(attribute.Value, nfi);
                        break;
                    case "normal2normal":
                         normal2normal= float.Parse(attribute.Value, nfi);
                        break;
                    case "normal2fast":
                        normal2fast = float.Parse(attribute.Value, nfi);
                        break;

                    case "fast2normal":
                        fast2normal = float.Parse(attribute.Value, nfi);
                        break;
                    case "fast2fast":
                        fast2fast = float.Parse(attribute.Value, nfi);
                        break;
                }
            }
        }
    }

    public class LinearGrammerModifier
    {
        static Random m_randomGenerator = new Random();

        protected LinearGrammerModifierDesc m_LgModParameter = new LinearGrammerModifierDesc();
        /// <summary>
        /// Parameter of linear grammer modifier
        /// </summary>
        public LinearGrammerModifierDesc LgModParameter
        {
            get { return m_LgModParameter; }
            set { m_LgModParameter = value; }
        }

        private float m_CurrentValue = 0.5f;
        public float CurrentValue
        {
            get { return m_CurrentValue; }
            set { m_CurrentValue = value; }
        }

        private SpeedState m_SpeedState = SpeedState.slow;

        public SpeedState SpeedState
        {
            get { return m_SpeedState; }
            set { m_SpeedState = value; }
        }

        private float m_Direction = 1;

        public float Direction
        {
            get { return m_Direction; }
            set { m_Direction = value; }
        }
        public LinearGrammerModifier()
        {
            m_CurrentValue = 0.5f;
            m_SpeedState = SpeedState.slow;
            m_Direction = 1;
        }
        public LinearGrammerModifier(LinearGrammerModifier original)
        {
            SetRandomParameter(original.m_LgModParameter);
            m_CurrentValue = original.CurrentValue;
            m_SpeedState = original.SpeedState;
            m_Direction = original.Direction;
        }
        public LinearGrammerModifier Clone()
        {
            return new LinearGrammerModifier(this);
        }
        public void SetRandomParameter(LinearGrammerModifierDesc lgModPar)
        {
            m_LgModParameter = lgModPar;
            m_CurrentValue = 0.5f;
            m_SpeedState = SpeedState.slow;
            m_Direction = 1;
        }
        public float GetValue()
        {
            return m_CurrentValue;
        }
        public void Update()
        {
            float rand = (float)m_randomGenerator.Next(1024) / 1024.0f;
            //float sum = 0;
            switch (m_SpeedState)
            {
                case SpeedState.slow:
                    m_CurrentValue += m_Direction * m_LgModParameter.stepSizeSlow;
                    if (rand < m_LgModParameter.slow2slow)
                    {
                    }
                    else if (rand < (m_LgModParameter.slow2slow + m_LgModParameter.slow2normal))
                    {
                        m_SpeedState = SpeedState.normal;
                    }
                    else
                    {
                        m_Direction *= -1;
                    }
                    break;
                case SpeedState.normal:
                    m_CurrentValue += m_Direction * m_LgModParameter.stepSizeNormal;
                    if (rand < m_LgModParameter.normal2slow)
                    {
                        m_SpeedState = SpeedState.slow;
                    }
                    else if (rand < (m_LgModParameter.normal2slow + m_LgModParameter.normal2normal))
                    {
                    }
                    else
                    {
                        m_SpeedState = SpeedState.fast;
                    }
                    break;
                case SpeedState.fast:
                    m_CurrentValue += m_Direction * m_LgModParameter.stepSizeFast;
                    if (rand < m_LgModParameter.fast2normal)
                    {
                        m_SpeedState = SpeedState.normal;
                    }
                    break;
            }
            // turn m_Direction at boundary
            if (m_CurrentValue < 0)
            {
                m_CurrentValue = 0;
                m_Direction *= -1;
            }
            if (m_CurrentValue > 1)
            {
                m_CurrentValue = 1;
                m_Direction *= -1;
            }
        }
    }
}
