/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

namespace Tgex
{
    public class RandomMovement : Modifier
    {
        Vector2Property position;
        IntProperty index;

        private LinearGrammerModifier m_XValue = new LinearGrammerModifier();
        public LinearGrammerModifier XValue
        {
            get { return m_XValue; }
            set { m_XValue = value; }
        }
        private LinearGrammerModifier m_YValue = new LinearGrammerModifier();
        public LinearGrammerModifier YValue
        {
            get { return m_YValue; }
            set { m_YValue = value; }
        }

        List<LinearGrammerModifierDesc> parameterList = new List<LinearGrammerModifierDesc>();
        public override void AddParameter(InstanceParameter parameter)
        {
            if (parameter.type == typeof(ParameterArray))
            {
                LinearGrammerModifierDesc desc = new LinearGrammerModifierDesc();
                ParameterArray parameterArray = (ParameterArray)parameter.value;
                foreach (InstanceParameter p in parameterArray.List)
                {
                    AddLgModParameter(p, desc);
                }
                parameterList.Add(desc);
            }
            else if (parameter.name == "Active")
            {
                Active = (bool)parameter.value;
                AddInitialParameter(parameter);
            }
        }

        private void AddLgModParameter(InstanceParameter parameter, LinearGrammerModifierDesc desc)
        {
            switch (parameter.name)
            {
                case "stepSizeSlow":
                    desc.stepSizeSlow = (float)parameter.value; 
                    break;
                case "stepSizeNormal":
                    desc.stepSizeNormal = (float)parameter.value;
                    break;
                case "stepSizeFast":
                    desc.stepSizeFast = (float)parameter.value;
                    break;

                case "slow2slow":
                    desc.slow2slow = (float)parameter.value;
                    break;
                case "slow2normal":
                    desc.slow2normal = (float)parameter.value;
                    break;
                case "slow2turn":
                    desc.slow2turn = (float)parameter.value;
                    break;

                case "normal2slow":
                    desc.normal2slow = (float)parameter.value;
                    break;
                case "normal2normal":
                    desc.normal2normal = (float)parameter.value;
                    break;
                case "normal2fast":
                    desc.normal2fast = (float)parameter.value;
                    break;

                case "fast2normal":
                    desc.fast2normal = (float)parameter.value;
                    break;
                case "fast2fast":
                    desc.fast2fast = (float)parameter.value;
                    break;
            }
        }

        public void SetRandomParameter(LinearGrammerModifierDesc lgModPar)
        {
            m_XValue.SetRandomParameter(lgModPar);
            m_YValue.SetRandomParameter(lgModPar);
        }

        public void SetRandomParameter()
        {
            int level = index.Value;
            if (level > parameterList.Count - 1) level = parameterList.Count - 1;
            m_XValue.SetRandomParameter(parameterList[level]);
            m_YValue.SetRandomParameter(parameterList[level]);
        }

        public override void Update(GameTime time)
        {
            m_XValue.Update();
            m_YValue.Update();
            position.Value = new Vector2(m_XValue.GetValue(), m_YValue.GetValue());
        }
        public RandomMovement()
        {
            m_Name = "randomMovement";
        }
        public RandomMovement(RandomMovement original)
            : base(original)
        {
            m_Name = "randomMovement";
            m_XValue = original.XValue.Clone();
            m_YValue = original.m_YValue.Clone();
        }
        public override Modifier Clone()
        {
            return new RandomMovement(this);
        }
        public override void Initialize()
        {
            base.Initialize();
            position = (Vector2Property)dynob.GetPropertyByName("position");
            index = (IntProperty)dynob.GetPropertyByName("currentLevel");
            SetRandomParameter();
        }
        public override void Reset()
        {
            base.Reset();
            SetRandomParameter();
        }
    }
}
