/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

namespace Tgex
{
    public class Module : IParametrizable
    {
        protected String m_Name;
        /// <summary>
        /// Identifier name for modifier.
        /// </summary>
        public String Name
        {
            get { return m_Name; }
            set { m_Name = value; }
        }

        private IAction finishAction = null;
        /// <summary>
        /// Action to be called when module has finished - optional.
        /// </summary>
        public IAction FinishAction
        {
            get { return finishAction; }
            set { finishAction = value; }
        }

        private bool active = true;

        public bool Active
        {
            get { return active; }
            set { active = value; }
        }

        protected Dynob dynob;
        protected List<InstanceParameter> m_initialParameterList = new List<InstanceParameter>();

        public virtual void AddParameter(InstanceParameter parameter)
        {
        }

        public virtual void Update(GameTime time)
        {
            // empty
        }

        virtual public void Initialize()
        {
        }

        virtual public void OnStart()
        {
        }

        public virtual void Reset()
        {
            foreach (InstanceParameter parameter in m_initialParameterList)
            {
                AddParameter(parameter);
            }
        }
        protected void AddInitialParameter(InstanceParameter parameter)
        {
            if (!m_initialParameterList.Exists(p => p.name == parameter.name))
            {
                m_initialParameterList.Add(parameter);
            }
        }
        public void Register(Dynob dynob)
        {
            this.dynob = dynob;
        }

        public void Finish()
        {
            active = false;
            if (finishAction != null)
            {
                finishAction.Execute();
            }
        }
        public virtual void Draw(GraphicsDeviceManager graphics)
        {
        }
    }
}
