/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace Tgex
{
    public class SpriteAnimationModule : SpriteModule
    {
        protected List<Texture2D> m_textures = new List<Texture2D>();
        protected List<String> m_textureNames = new List<string>();

        int currentTexture = 0;
        double changeTextureInterval = 100;
        double timeSinceLastTexture = 0;
        static Random random = new Random();
        float alpha = 0;
        private bool animate = true;

        public bool Animate
        {
            get { return animate; }
            set { animate = value; }
        }

        public void SelectTexture(int index)
        {
            if (index < m_textures.Count)
            {
                currentTexture = index;
            }
        }

        bool repeat = true;

        public SpriteAnimationModule(bool repeat)
            : base()
        {
            this.repeat = repeat;
        }
        public SpriteAnimationModule()
            : base()
        {
        }
        public override void Initialize()
        {
            base.Initialize();
            position = (Vector2Property)dynob.GetPropertyByName("position");
            //rotation = (Vector2Property)dynob.GetPropertyByName("rotation");
            //scale = (Vector2Property)dynob.GetPropertyByName("scale");
            m_sprite = new SpriteBatch(DynobFactory.Instance.GraphicsManager.GraphicsDevice);
            foreach (String textureName in m_textureNames)
            {
                if (textureName != null)
                {
                    m_textures.Add(DynobFactory.Instance.ContentManager.Load<Texture2D>(textureName));
                }
            }
        }
        public override void Update(GameTime time)
        {
            base.Update(time);

            if (animate)
            {
                timeSinceLastTexture += time.ElapsedGameTime.TotalMilliseconds;
                if (timeSinceLastTexture > changeTextureInterval)
                {
                    timeSinceLastTexture = timeSinceLastTexture - changeTextureInterval;
                    currentTexture++;
                    if (currentTexture >= m_textures.Count)
                    {
                        currentTexture = 0;
                        if (!repeat)
                        {
                            this.Finish();
                        }
                    }
                    //int newVal;
                    //do
                    //{
                    //    newVal = SpriteAnimationModule.random.Next(m_textures.Count);
                    //}
                    //while (currentTexture == newVal);
                    //currentTexture = newVal;

                    //alpha += 5.0f*(float)Math.PI / (float)time.ElapsedGameTime.TotalMilliseconds;
                }
            }            
        }
        public override void Draw(GraphicsDeviceManager graphics)
        {
            // calc size
            viewportSize = new Vector2((float)graphics.GraphicsDevice.Viewport.Width,
                                        (float)graphics.GraphicsDevice.Viewport.Height);
            pixelSize = new Vector2(1 / viewportSize.X, 1 / viewportSize.Y);

            float textureWidth = m_textures[currentTexture].Width * pixelSize.X;
            float textureHeight = m_textures[currentTexture].Height * pixelSize.Y;
            Vector2 origTexSize = new Vector2(m_textures[currentTexture].Width,
                                         m_textures[currentTexture].Height);
            Vector2 curTexSize;
            if (targetSize.X == 0)
            {
                curTexSize = origTexSize;
            }
            else
            {
                curTexSize = targetSize;
            }

            // it dies after it is out of the screen
            if ((position.Value.X > 1) || (position.Value.X < 0)
                || (position.Value.Y > 1) || (position.Value.Y < 0))
            {
                dynob.Dead = true;
            }
            else
            {
                if (m_sprite != null)
                {
                    m_sprite.Begin(SpriteBlendMode.AlphaBlend);
                    m_spritePosition.X = (position.Value.X ) * viewportSize.X;
                    m_spritePosition.Y = (position.Value.Y ) * viewportSize.Y;
                    Rectangle rect = new Rectangle();
                    rect.X = (int)m_spritePosition.X;
                    rect.Y = (int)m_spritePosition.Y;
                    rect.Height = (int)curTexSize.X;
                    rect.Width = (int)curTexSize.Y;
                    m_sprite.Draw(m_textures[currentTexture], rect, null, Color.White, alpha,
                        origTexSize/2.0f, SpriteEffects.None, 0.1f);
                    m_sprite.End();
                }
            }
        }
        public override void AddParameter(InstanceParameter parameter)
        {
            switch (parameter.name)
            {
                case "FinishAction":
                    FinishAction = (IAction)parameter.value;
                    break;
                case "Active":
                    Active = (bool)parameter.value;
                    AddInitialParameter(parameter);
                    break;
                case "Repeat":
                    repeat = (bool)parameter.value;
                    AddInitialParameter(parameter);
                    break;
                case "TargetSize":
                    targetSize = (Vector2)parameter.value;
                    break;
                case "AnimationTexture":
                    m_textureNames.Add((string)parameter.value);
                    break;
            }
        }
        public override void InitVisualization(GraphicsDeviceManager graphics, ContentManager contentManager)
        {
        }
        public void AddTexture(String textureName, ContentManager contentManager)
        {
            m_textures.Add(contentManager.Load<Texture2D>(textureName));
        }
    }
}
