/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace Tgex
{
    public class SpriteModule : Module
    {
        protected bool m_visible = true;

        protected SpriteBatch m_sprite;
        protected Texture2D m_texture;
        protected String m_textureName;
        protected Vector2 m_spritePosition = new Vector2();
        //protected ContentManager m_content;
        //protected GraphicsDeviceManager m_graphics;
        protected Vector2Property position;
        protected Vector2Property rotation;
        protected Vector2Property scale;

        protected FloatProperty spriteRotation;

        protected Vector2 viewportSize;
        protected Vector2 pixelSize;

        protected Vector2 targetSize;

        public Vector2 TargetSize
        {
            get { return targetSize; }
            set { targetSize = value; }
        }

        public SpriteModule() : base()
        {
        }
        public override void Initialize()
        {
            base.Initialize();
            position = (Vector2Property)dynob.GetPropertyByName("position");
            spriteRotation = (FloatProperty)dynob.GetPropertyByName("spriteRotation");
            //rotation = (Vector2Property)dynob.GetPropertyByName("rotation");
            //scale = (Vector2Property)dynob.GetPropertyByName("scale");
            m_sprite = new SpriteBatch(DynobFactory.Instance.GraphicsManager.GraphicsDevice);
            if (m_textureName != null)
            {
                m_texture = DynobFactory.Instance.ContentManager.Load<Texture2D>(m_textureName);
            }
        }
        public override void Update(GameTime time)
        {
            base.Update(time);
        }
        public override void Draw(GraphicsDeviceManager graphics)
        {
            // calc size
            viewportSize = new Vector2((float)graphics.GraphicsDevice.Viewport.Width,
                                        (float)graphics.GraphicsDevice.Viewport.Height);
            pixelSize = new Vector2(1 / viewportSize.X, 1 / viewportSize.Y);

            float textureWidth = m_texture.Width * pixelSize.X;
            float textureHeight = m_texture.Height * pixelSize.Y;
            Vector2 origTexSize = new Vector2(m_texture.Width,
                                              m_texture.Height);
            Vector2 curTexSize;
            if (targetSize.X == 0)
            {
                curTexSize = origTexSize;
            }
            else
            {
                curTexSize = targetSize;
            }


            // it dies after it is out of the screen
            if ((position.Value.X > 1) || (position.Value.X < 0)
                || (position.Value.Y > 1) || (position.Value.Y < 0))
            {
                dynob.Dead = true;
            }
            else
            {
                if (m_sprite != null)
                {
                    m_sprite.Begin(SpriteBlendMode.AlphaBlend);
                    m_spritePosition.X = (position.Value.X) * viewportSize.X;
                    m_spritePosition.Y = (position.Value.Y) * viewportSize.Y;
                    Rectangle rect = new Rectangle();
                    rect.X = (int)m_spritePosition.X;
                    rect.Y = (int)m_spritePosition.Y;
                    rect.Height = (int)curTexSize.X;
                    rect.Width = (int)curTexSize.Y;
                    float angle = 0;
                    if (spriteRotation != null) angle = spriteRotation.Value;
                    m_sprite.Draw(m_texture, rect, null, Color.White,
                        angle,
                        origTexSize / 2.0f, SpriteEffects.None, 0.1f);
                    m_sprite.End();
                }
            }
        }
        public override void AddParameter(InstanceParameter parameter)
        {
            switch (parameter.name)
            {
                case "FinishAction":
                    FinishAction = (IAction)parameter.value;
                    break;
                case "Active":
                    Active = (bool)parameter.value;
                    AddInitialParameter(parameter);
                    break;
                case "TargetSize":
                    targetSize = (Vector2)parameter.value;
                    break;
                case "Texture":
                    m_textureName = (string)parameter.value;
                    break;
            }
        }
        public virtual void InitVisualization(GraphicsDeviceManager graphics, ContentManager contentManager)
        {
            //m_sprite = new SpriteBatch(graphics.GraphicsDevice);
            //if (m_textureName != null)
            //{
            //    m_texture = contentManager.Load<Texture2D>(m_textureName);
            //}
        }
        public void LoadTexture(String textureName, ContentManager contentManager)
        {
            m_texture = contentManager.Load<Texture2D>(textureName);
        }

    }
}
