﻿/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tgex
{
    public class RecycleList<T> where T : IRecyclable, new()
    {
        Stack<T> m_recycleStack;
        protected List<T> m_Items;
        /// <summary>
        /// List of the active items.
        /// </summary>
        public List<T> Items
        {
            get { return m_Items; }
            set { m_Items = value; }
        }

        public RecycleList()
        {
            m_Items = new List<T>();
            m_recycleStack = new Stack<T>();
        }

        public void RemoveAll(Predicate<T> condition)
        {
            List<T> result = m_Items.FindAll(condition);
            m_Items.RemoveAll(condition);
            result.ForEach(item => { item.Recycle(); m_recycleStack.Push(item); });
        }

        public void Remove(T item)
        {
            m_Items.Remove(item);
            item.Recycle();
            m_recycleStack.Push(item);
        }

        public T CreateItem()
        {
            T item = default(T);
            if (m_recycleStack.Count > 0)
            {
                item = m_recycleStack.Pop();
            }
            else
            {
                item = new T();
            }
            m_Items.Add(item);
            return item;
        }
    }
}
