/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Xml;
using System.Globalization;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;
using Tgex.Graphics;

namespace Tgex
{
    /// <summary>
    /// Available Stereo modes
    /// </summary>
    public enum StereoViewMode
    {
        None,
        FieldInterlaced,
        LineInterlaced,
        HorizontalSpan,
        VerticalSpan
    }

    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class VRGame : Microsoft.Xna.Framework.Game
    {
        private RenderWindow m_renderWindow = null;

        public RenderWindow RenderWindow
        {
            get { return m_renderWindow; }
            set { m_renderWindow = value; }
        }

        private GameSettings m_gameSettings = new GameSettings();

        public GameSettings GameSettings
        {
            get { return m_gameSettings; }
            set { m_gameSettings = value; }
        }

        protected GraphicsDeviceManager m_graphicsManager;
        public GraphicsDeviceManager GraphicsManager
        {
            get { return m_graphicsManager; }
        }

        protected StereoViewMode m_stereoMode;
        /// <summary>
        /// Stereo mode of camera
        /// </summary>
        public StereoViewMode StereoMode
        {
            get { return m_stereoMode; }
            set { m_stereoMode = value; }
        }

        protected Camera m_camera;

        public Camera Camera
        {
            get { return m_camera; }
            set { m_camera = value; }
        }

        private static VRGame instance;
        public static VRGame Instance
        { get { return instance; } }

        public VRGame()
            : base()
        {
            instance = this;
            m_camera = new Camera();
            m_camera.Initialize();
            Content.RootDirectory = "Content";

            // read settings file
            m_gameSettings.ReadSettings();
            m_stereoMode = m_gameSettings.stereoMode;
            m_camera.StereoMode = m_stereoMode;
            m_camera.FieldOfView = m_gameSettings.fieldOfView;
            m_camera.EyeDistance = m_gameSettings.eyeDistance;

            // create graphics manager
            m_graphicsManager = new GraphicsDeviceManager(this);

            // Set minimum requirements
            m_graphicsManager.MinimumPixelShaderProfile = ShaderProfile.PS_2_0;
            m_graphicsManager.MinimumVertexShaderProfile = ShaderProfile.VS_2_0;

            // create render window
            switch (m_stereoMode)
            {
                case StereoViewMode.HorizontalSpan:
                    m_renderWindow = new HorizontalSpanRenderWindow(this);
                    break;
                case StereoViewMode.LineInterlaced:
                    m_renderWindow = new LineInterlacedRenderWindow(this);
                    break;
            }
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();

            m_renderWindow.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here

            base.Update(gameTime);
        }

        /// <summary>
        /// Draw
        /// </summary>
        /// <param name="gameTime">Game time</param>
        protected override void Draw(GameTime gameTime)
        {
            m_renderWindow.Draw(gameTime);

            base.Draw(gameTime);
        }

        /// <summary>
        /// Override this one for drawing
        /// </summary>
        /// <param name="gameTime"></param>
        public virtual void DrawScene(GameTime gameTime)
        {
        }
    }
}