﻿/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using Microsoft.Xna.Framework;
using Tgex;

namespace Tvrx
{
    static public class ConfigReader
    {
        public static List<CaptureCameraDesc> ReadCameraDesc()
        {
            List<CaptureCameraDesc> cameraDescList = new List<CaptureCameraDesc>();
            // read settings
            // get config from xml file
            System.Globalization.NumberFormatInfo nfi
                = System.Globalization.CultureInfo.InvariantCulture.NumberFormat;
            try
            {
                XmlDocument xmlFile = new XmlDocument();
                xmlFile.Load("./Config/cameras.xml");
                XmlNodeList cameraNodes = xmlFile.GetElementsByTagName("camera");
                foreach (XmlNode cameraNode in cameraNodes)
                {
                    CaptureCameraDesc cameraDesc = new CaptureCameraDesc();
                    cameraDesc.id = cameraNode.Attributes.GetNamedItem("model").Value;
                    foreach (XmlNode child in cameraNode.ChildNodes)
                    {
                        switch (child.Name)
                        {
                            case "model":
                                cameraDesc.model = child.InnerText;
                                break;
                            case "sizeX":
                                cameraDesc.width = Int32.Parse(child.InnerText);
                                break;
                            case "sizeY":
                                cameraDesc.height = Int32.Parse(child.InnerText);
                                break;
                            case "pixelSize":
                                cameraDesc.pixelSize = float.Parse(child.InnerText, nfi);
                                break;
                            case "focus_lenX":
                                cameraDesc.focalLengthX = float.Parse(child.InnerText, nfi);
                                break;
                            case "focus_lenY":
                                cameraDesc.focalLengthY = float.Parse(child.InnerText, nfi);
                                break;
                            case "PrincipalX":
                                cameraDesc.principalPointX = float.Parse(child.InnerText, nfi);
                                break;
                            case "PrincipalY":
                                cameraDesc.principalPointY = float.Parse(child.InnerText, nfi);
                                break;
                            case "Dist1":
                                cameraDesc.k1 = float.Parse(child.InnerText, nfi);
                                break;
                            case "Dist2":
                                cameraDesc.k2 = float.Parse(child.InnerText, nfi);
                                break;
                            case "Dist3":
                                cameraDesc.p1 = float.Parse(child.InnerText, nfi);
                                break;
                            case "Dist4":
                                cameraDesc.p2 = float.Parse(child.InnerText, nfi);
                                break;
                            case "ColorMode":
                                cameraDesc.SetColorMode(child.InnerText);
                                break;
                        }
                    }
                    cameraDesc.flipImage = false;
                    cameraDescList.Add(cameraDesc);
                }
            }
            catch (Exception)
            {
                //System.Windows.Forms.MessageBox.Show("Error reading config file!");
            }
            return cameraDescList;
        }

        public static void ReadTrackerConfig(
            out List<MarkerBodyDesc> markerDescList,
            out List<MarkerTrackerDesc> markerTrackerDescList,
            out List<TrackedDeviceDesc> trackedDeviceDescList,
            out List<TrackerProxyDesc> trackerProxyDescList,
            out List<DataFilterDesc> dataFilterDescList)
        {
            markerDescList = new List<MarkerBodyDesc>();
            markerTrackerDescList = new List<MarkerTrackerDesc>();
            trackedDeviceDescList = new List<TrackedDeviceDesc>();
            trackerProxyDescList = new List<TrackerProxyDesc>();
            dataFilterDescList = new List<DataFilterDesc>();
            System.Globalization.NumberFormatInfo nfi
                = System.Globalization.CultureInfo.InvariantCulture.NumberFormat;
            // read settings
            // get config from xml file
            try
            {
                XmlDocument xmlFile = new XmlDocument();
                xmlFile.Load("./Config/tracker.xml");
                //
                // read tracker cameras
                //
                XmlNodeList trackerNodes = xmlFile.GetElementsByTagName("TrackerCam");
                foreach (XmlNode trackerNode in trackerNodes)
                {
                    MarkerTrackerDesc mtd = new MarkerTrackerDesc();
                    mtd.Id = Int32.Parse(trackerNode.Attributes.GetNamedItem("id").Value);
                    mtd.CameraModel = trackerNode.Attributes.GetNamedItem("cameramodel").Value;
                    mtd.Translation = trackerNode.Attributes.GetNamedItem("translation").Value.ParseVector3();
                    mtd.Rotation = trackerNode.Attributes.GetNamedItem("rotation").Value.ParseVector3();
                    mtd.Scale = float.Parse(trackerNode.Attributes.GetNamedItem("scale").Value, nfi);
                    mtd.XAxis = trackerNode.Attributes.GetNamedItem("xAxis").Value;
                    mtd.YAxis = trackerNode.Attributes.GetNamedItem("yAxis").Value;
                    mtd.ZAxis = trackerNode.Attributes.GetNamedItem("zAxis").Value;
                    markerTrackerDescList.Add(mtd);
                }
                //
                // read tracker
                //
                trackerNodes = xmlFile.GetElementsByTagName("TrackedDevice");
                foreach (XmlNode trackerNode in trackerNodes)
                {
                    String typeString = trackerNode.Attributes.GetNamedItem("type").Value;
                    TrackedDeviceDesc tdd = null;
                    if (typeString == "TCAM")
                    {
                        TrackedMarkerDesc tmd = new TrackedMarkerDesc();
                        tdd = tmd;
                        foreach (XmlNode child in trackerNode.ChildNodes)
                        {
                            if (child.Name == "MarkerBodyId")
                            {
                                tmd.MarkerBodyId = Int32.Parse(child.InnerText);
                            }
                            else if (child.Name == "TCamId")
                            {
                                tmd.TCamId = Int32.Parse(child.InnerText);
                            }
                        }                        
                    }
                    if (typeString == "WiiMote")
                    {
                        TrackedWiiMoteDesc tmd = new TrackedWiiMoteDesc();
                        tdd = tmd;
                        foreach (XmlNode child in trackerNode.ChildNodes)
                        {
                            if (child.Name == "MarkerBodyId")
                            {
                                tmd.MarkerBodyId = Int32.Parse(child.InnerText);
                            }
                        }
                    }
                    foreach (XmlNode child in trackerNode.ChildNodes)
                    {
                        if (child.Name == "LocalTranslation")
                        {
                            tdd.LocalTranslation = child.InnerText.ParseVector3();
                        }
                        else if (child.Name == "LocalRotation")
                        {
                            tdd.LocalRotation = child.InnerText.ParseVector3();
                        }
                        else if (child.Name == "WorldTranslation")
                        {
                            tdd.WorldTranslation = child.InnerText.ParseVector3();
                        }
                        else if (child.Name == "WorldRotation")
                        {
                            tdd.WorldRotation = child.InnerText.ParseVector3();
                        }
                        else if (child.Name == "RotationFilterId")
                        {
                            tdd.RotationFilterId = Int32.Parse(child.InnerText);
                        }
                        else if (child.Name == "TranslationFilterId")
                        {
                            tdd.TranslationFilterId = Int32.Parse(child.InnerText);
                        }
                        else if (child.Name == "ReverseTranslation")
                        {
                            String[] values = child.InnerText.Split(',');
                            for (int i = 0; i < 3; i++) tdd.ReverseTranslation[i] = bool.Parse(values[i]);
                        }
                        else if (child.Name == "ReverseRotation")
                        {
                            String[] values = child.InnerText.Split(',');
                            for (int i = 0; i < 3; i++) tdd.ReverseRotation[i] = bool.Parse(values[i]);
                        }
                        else if (child.Name == "TrackerCamId")
                        {
                            tdd.TrackerCamId = Int32.Parse(child.InnerText);
                        }
                    }
                    tdd.Id = Int32.Parse(trackerNode.Attributes.GetNamedItem("id").Value);
                    tdd.HasTranslation = bool.Parse(trackerNode.Attributes.GetNamedItem("translation").Value);
                    tdd.HasRotation = bool.Parse(trackerNode.Attributes.GetNamedItem("rotation").Value);
                    trackedDeviceDescList.Add(tdd);
                }
                //
                // read tracker proxies
                //
                trackerNodes = xmlFile.GetElementsByTagName("TrackerProxy");
                foreach (XmlNode trackerNode in trackerNodes)
                {
                    TrackerProxyDesc tpd = new TrackerProxyDesc();
                    tpd.Id = Int32.Parse(trackerNode.Attributes.GetNamedItem("id").Value);
                    foreach (XmlNode child in trackerNode.ChildNodes)
                    {
                        if (child.Name == "TranslationDeviceIds")
                        {
                            String[] values = child.InnerText.Split(',');
                            for (int i = 0; i < values.Count(); i++)
                            {
                                tpd.TranslationDeviceIds.Add(Int32.Parse(values[i]));
                            }
                        }
                        else if (child.Name == "RotationDeviceIds")
                        {
                            String[] values = child.InnerText.Split(',');
                            for (int i = 0; i < values.Count(); i++)
                            {
                                tpd.RotationDeviceIds.Add(Int32.Parse(values[i]));
                            }
                        }
                    }
                    trackerProxyDescList.Add(tpd);
                }
                //
                // read marker bodies
                //
                XmlNodeList markerNodes = xmlFile.GetElementsByTagName("MarkerBody")[0].ChildNodes;
                MarkerBodyDesc markerDesc = null;
                foreach (XmlNode markerNode in markerNodes)
                {
                    if (markerNode.Name == "TCamMarkerBody")
                    {
                        TCamMarkerBodyDesc tcamMarkerDesc = new TCamMarkerBodyDesc();
                        tcamMarkerDesc.IdMarker = float.Parse(markerNode.Attributes.GetNamedItem("idMarker").Value, nfi);
                        tcamMarkerDesc.CrossPoint = float.Parse(markerNode.Attributes.GetNamedItem("crossPoint").Value, nfi);
                        tcamMarkerDesc.BarLength = float.Parse(markerNode.Attributes.GetNamedItem("barLength").Value, nfi);
                        markerDesc = tcamMarkerDesc;
                    }
                    else if (markerNode.Name == "WiiMarkerBody")
                    {
                        // wiimote has no additional parameters
                        markerDesc = new MarkerBodyDesc();
                    }
                    markerDesc.Name = markerNode.Attributes.GetNamedItem("name").Value;
                    markerDesc.Id = Int32.Parse(markerNode.Attributes.GetNamedItem("id").Value);
                    markerDesc.NearClip = float.Parse(markerNode.Attributes.GetNamedItem("nearClip").Value, nfi);
                    markerDesc.FarClip = float.Parse(markerNode.Attributes.GetNamedItem("farClip").Value, nfi);
                    // create local transform
                    Vector3 translation = markerNode.Attributes.GetNamedItem("translation").Value.ParseVector3();
                    Vector3 rotation = markerNode.Attributes.GetNamedItem("rotation").Value.ParseVector3();
                    Matrix trans =
                        Matrix.CreateFromYawPitchRoll(rotation.Y, rotation.X, rotation.Z)
                        * Matrix.CreateTranslation(translation);

                    foreach (XmlNode child in markerNode.ChildNodes)
                    {
                        if (child.Name == "point3d")
                        {
                            Vector3 point = new Vector3();
                            // transform points according to local transform of marker body
                            point = Vector3.Transform(
                                        point.TryParse(child.Attributes.GetNamedItem("value").Value),
                                        trans);
                            markerDesc.WorldMarker.Add(point);
                        }
                    }
                    markerDescList.Add(markerDesc);
                }
                //
                // read filter
                //
                XmlNodeList filterNodes = xmlFile.GetElementsByTagName("Datafilter")[0].ChildNodes;
                DataFilterDesc dataFilterDesc = null;
                foreach (XmlNode filterNode in filterNodes)
                {
                    // only support kalman by now
                    if (filterNode.Name == "Kalman")
                    {
                        KalmanFilterDesc kalmanDesc = new KalmanFilterDesc();
                        String[] A = filterNode.Attributes.GetNamedItem("A").Value.Split(',');
                        for (int i = 0; i < 4; i++)
                        {
                            kalmanDesc.A[i] = float.Parse(A[i], nfi);
                        }
                        kalmanDesc.measurement_noise_cov =
                            float.Parse(filterNode.Attributes.GetNamedItem("measurement_noise_cov").Value, nfi);
                        kalmanDesc.process_noise_cov_1 =
                            float.Parse(filterNode.Attributes.GetNamedItem("process_noise_cov_1").Value, nfi);
                        kalmanDesc.process_noise_cov_2 =
                            float.Parse(filterNode.Attributes.GetNamedItem("process_noise_cov_2").Value, nfi);
                        dataFilterDesc = kalmanDesc;

                        dataFilterDesc.id = Int32.Parse(filterNode.Attributes.GetNamedItem("id").Value);
                        dataFilterDesc.className = filterNode.Attributes.GetNamedItem("class").Value;
                        dataFilterDescList.Add(dataFilterDesc);
                    }
                }
            }
            catch (Exception)
            {
                //System.Windows.Forms.MessageBox.Show("Error reading config file!");
                ErrorHandler.Report("Error parsing tracker config file!");
            }
        }
    }
}
