﻿/*
 * Copyright (C): Timo Fleisch
 *                2009 Berlin, Germany
 *                http://www.vrhome.de
 *                All Rights Reserved.
 *
 * THIS SOURCECODE IS PUBLISHED UNDER RESTRICTIONS.
 * No unauthorized copy or distribution allowed.
 * Please see website for detailed liscence information.
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Emgu.Util;
using Emgu.CV;
using System.Runtime.InteropServices;

namespace Tvrx
{
    public class KalmanFilter : IDataFilter
    {
        private KalmanFilterDesc filterDesc;
        private Kalman m_kalman;

        public KalmanFilter()
        {
            m_kalman = new Kalman(2, 1, 0);
        }

        public void SetFilterDesc(DataFilterDesc desc)
        {
            filterDesc = (KalmanFilterDesc)desc;
            
	        // set A - second parameter is frames per second
            m_kalman.TransitionMatrix.Data.SetValue(filterDesc.A[0], 0, 0);
            m_kalman.TransitionMatrix.Data.SetValue(filterDesc.A[1], 0, 1);
            m_kalman.TransitionMatrix.Data.SetValue(filterDesc.A[2], 1, 0);
            m_kalman.TransitionMatrix.Data.SetValue(filterDesc.A[3], 1, 1);

	        // set H
            m_kalman.MeasurementMatrix.Data.SetValue(1.0f, 0, 0);
            m_kalman.MeasurementMatrix.Data.SetValue(0.0f, 0, 1);

	        // set Q
            CvInvoke.cvSetIdentity(m_kalman.ProcessNoiseCovariance.Ptr, new MCvScalar(1));
            m_kalman.ProcessNoiseCovariance.Data.SetValue(filterDesc.process_noise_cov_1, 0, 0);
            m_kalman.ProcessNoiseCovariance.Data.SetValue(filterDesc.process_noise_cov_2, 1, 0);

	        // set R - assume measurement error sigma is one pixel size 0.00465
            CvInvoke.cvSetIdentity(m_kalman.MeasurementNoiseCovariance.Ptr, new MCvScalar(1e-5));
            m_kalman.MeasurementNoiseCovariance.Data.SetValue(filterDesc.measurement_noise_cov, 0, 0);

            CvInvoke.cvSetIdentity(m_kalman.ErrorCovariancePost.Ptr, new MCvScalar(500));
            m_kalman.ErrorCovariancePost.Data.SetValue(2, 0, 0);            
        }

        Matrix<float> data = new Matrix<float>(new float[] { 0 });
        Matrix<float> predict = new Matrix<float>(new float[] { 0 });
        public float Filter(float inData)
        {            
            // Z measurement
            data.Data[0,0] = inData;
            m_kalman.Predict(predict);
            m_kalman.Correct(data);
            //data.Dispose();
            //predict.Dispose();
            return m_kalman.CorrectedState[0, 0];
        }
    }
}
