using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Tgex.Graphics;
using Tgex;
using Tvrx;

namespace VRDesktopDemo
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class VRDesktop : VRGame
    {
        // transformation matrix for the hand tracked model
        Matrix modelTransform = Matrix.Identity;
        // model date for the hand tracked model
        Model model;
        Matrix[] modelTransforms;

        public VRDesktop() : base()
        {
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // The tracker manager is a singleton but needs to be initialized once.
            TrackerManager.Instance.Initialize();
            // initialize base clase.
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // for this demo just load the coordinate cross
            model = Content.Load<Model>("coordinate");
            modelTransforms = new Matrix[model.Bones.Count];
            // start tracking now
            TrackerManager.Instance.StartTracking();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if ((GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                    || (Keyboard.GetState().IsKeyDown(Keys.Escape)))
            {
                TrackerManager.Instance.StopTracking();
                this.Exit();
            }
            TrackerManager.Instance.Update();
            // tracking proxy with id 1 is the hand tracker
            modelTransform = TrackerManager.Instance.GetProxyTransform(1);
            // tracking proxy with id 0 is the head tracker - change eye position.
            m_camera.EyePosition = TrackerManager.Instance.GetProxyTransform(0).Translation;

            base.Update(gameTime);
        }

        public override void DrawScene(GameTime gameTime)
        {
            GraphicsDevice.Clear(ClearOptions.Target | ClearOptions.DepthBuffer,
                Color.MidnightBlue, 1.0f, 0);

            //Copy any parent transforms
            model.CopyAbsoluteBoneTransformsTo(modelTransforms);

            //Draw the model, a model can have multiple meshes, so loop
            foreach (ModelMesh mesh in model.Meshes)
            {
                //This is where the mesh orientation is set, as well as our camera and projection
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    effect.World = modelTransforms[mesh.ParentBone.Index]
                        * Matrix.CreateScale(0.01f) // coordinate scale
                        * modelTransform;
                    effect.View = m_camera.ViewMatrix;
                    effect.Projection = m_camera.ProjectionMatrix;
                }
                //Draw the mesh, will use the effects set above.
                mesh.Draw();
            }
        }
    }
}
